program hist_edd
  
  implicit none
  
  character*80 :: input = 'zeitreihe.dat'
  character*80 :: output1 = 'drift_20.dat'
  character*80 :: output2 = 'diff_20.dat'
  real*8,parameter :: delt = 0.01d0
  integer,parameter :: nbin = 20
  integer,parameter :: tau_max = 1
  integer,parameter :: every_tau = 1
  
  integer :: ndp, tau
  real*8 :: h, mini, maxi, dx, y, y2, x0, taudt
  real*8, allocatable :: x(:)
  real*8,dimension(nbin,tau_max) :: d1, d2, m4, d1_err, d2_err
  integer,dimension(nbin,tau_max) :: ndp_bin
  real*8,dimension(nbin) :: axis
  integer :: i,j,k,ios
  
  print*, ' '
  print*, 'Lese Daten ...'
  
  open(10,file=input,status='old')

  ndp = -1
  ios = 0
  
  do while (ios==0)
    read(10,*,iostat=ios) x0
    ndp=ndp+1 
  end do
  
  rewind(10)
  
  print*, ndp, 'Datenpunkte'
  
  allocate(x(ndp))
  
  do i=1,ndp
    read(10,*) x(i)
  end do
  
  close(10)
  print*, 'Fertig.'
  
  print*, ''
  print*, 'Bestimme Wertebereich ...'
  
  mini = x(1)
  maxi = x(1)
  
  do i=2,ndp
    if ( x(i)<mini ) mini = x(i)
    if ( x(i)>maxi ) maxi = x(i)
  end do
  
  mini = mini - (maxi-mini)*0.1d0
  maxi = maxi + (maxi-mini)*0.1d0
  
  print*, 'Minimum:', mini
  print*, 'Maximum:', maxi
  print*, ''
  print*, 'Bestimme Koordinatenachse ...'
  
  dx = (maxi - mini)/real(nbin,8)
  
  do i=1,nbin
    axis(i) = mini + ( 0.5d0 + real(i-1,8) )*dx
  end do
  
  print*, ''
  print*, 'Bestimme Drift und Diffusion ...'

  d1 = 0.0d0
  d2 = 0.0d0
  m4 = 0.0d0
  ndp_bin = 0
  
  do tau = every_tau,tau_max,every_tau
  
    print*, 'tau', tau
    
    do j=1,ndp-tau
    
      i = int( (x(j) - mini)/dx ) + 1
      ndp_bin(i,tau) = ndp_bin(i,tau) + 1
      
      y = x(j+tau) - x(j)
      y2= y*y
      d1(i,tau) = d1(i,tau) + y
      d2(i,tau) = d2(i,tau) + y2
      m4(i,tau) = m4(i,tau) + y2*y2
      
    end do
    
    taudt = real(tau,8)*delt
        
    d1(:,tau) = d1(:,tau) / taudt
    d2(:,tau) = d2(:,tau) / (2.0d0*taudt)
    
  end do
  
  do i=1,nbin
    do j=every_tau,tau_max,every_tau
      d1(i,j) = d1(i,j) / real(ndp_bin(i,j),8)
      d2(i,j) = d2(i,j) / real(ndp_bin(i,j),8)
      m4(i,j) = m4(i,j) / real(ndp_bin(i,j),8)
    end do
  end do
      
  do i=1,nbin
    do j=every_tau,tau_max,every_tau
    
      if (ndp_bin(i,j) > 10) then
	taudt = real(j,8)*delt
	
	d1_err(i,j) = 2.0d0*d2(i,j)/taudt - d1(i,j)*d1(i,j)
	d1_err(i,j) = d1_err(i,j)/real(ndp_bin(i,j),8)
	d1_err(i,j) = sqrt(d1_err(i,j))
	
	d2_err(i,j) = 0.25d0*m4(i,j)/(taudt*taudt) - d2(i,j)*d2(i,j)
	d2_err(i,j) = d2_err(i,j) / real(ndp_bin(i,j),8)
	d2_err(i,j) = sqrt(d2_err(i,j))
      end if
      
    end do
  end do
  
  open(20,file=output1,status='replace')
  do j=every_tau,tau_max,every_tau
    do i=1,nbin
      if (ndp_bin(i,j)>10) then
	write(20,"(4(e13.6,x))") real(j,8)*delt, axis(i), d1(i,j), d1_err(i,j)
      end if
    end do
    write(20,*) ''
  end do
  close(20)
  
  open(30,file=output2,status='replace')
  do j=every_tau,tau_max,every_tau
    do i=1,nbin
      if (ndp_bin(i,j)>10) then
	write(30,"(4(e13.6,x))") real(j,8)*delt, axis(i), d2(i,j), d2_err(i,j)
      end if
    end do
    write(30,*) ''
  end do
  close(30)
  
end program

